==> 1.2.8-alpha.1.tar.gz: pos-be/src/modules/queue/infrastructure/handlers/transaction.handler.ts <==
import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { TransactionDataService } from 'src/modules/transaction/transaction/data/services/transaction-data.service';
import {
  TransactionChangeStatusEvent,
  TransactionCreateQueueEvent,
} from 'src/modules/transaction/transaction/domain/entities/event/transaction-change-status.event';
import { TicketDataService } from '../../data/services/ticket.service';
import { QueueOrder } from '../../domain/entities/order.entity';
import { QueueTicket } from '../../domain/entities/ticket.entity';
import { QueueItem } from '../../domain/entities/queue-item.entity';
import * as moment from 'moment';
import { TransactionUserType } from 'src/modules/transaction/transaction/constants';
import { RegisterQueueManager } from '../../domain/usecases/register-queue.manager';
import { RegisterQueueDto } from '../controllers/dto/register-queue.dto';
import { QueueService } from '../../data/services/queue.service';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';
import { QueueBucketReadService } from '../../data/services/queue-bucket';

@EventsHandler(TransactionChangeStatusEvent, TransactionCreateQueueEvent)
export class QueueTransactionHandler
  implements IEventHandler<TransactionChangeStatusEvent>
{
  constructor(
    private readonly dataService: TransactionDataService,
    private readonly ticketService: TicketDataService,
    private readonly queueService: QueueService,
    private readonly bucketService: QueueBucketReadService,
  ) {}

  async handle(event: TransactionChangeStatusEvent) {
    const process_data = event.data.data;

    /**
     * If data still in process (not settled) then don't create the queue order
     */
    if (process_data?.status != 'settled') return;

    const transaction = await this.dataService.getOneByOptions({
      where: {
        id: event.data.id,
      },
      relations: ['customer_category', 'items', 'items.item'],
    });

    const date = transaction.booking_date ?? transaction.invoice_date;
    const queue_date = moment(date, 'YYYY-MM-DD').unix();

    const { id, customer_name, customer_phone, invoice_code } = transaction;

    const customerOrder = {
      code: invoice_code,
      customer: customer_name,
      phone: customer_phone,
      date: queue_date * 1000,
      transaction_id: id,
    };

    const items = transaction.items
      .filter((item) => item.item.use_queue)
      .map<QueueItem>((item) => {
        return {
          item_queue_id: item.item.item_queue_id,
          item_id: item.item_id,
          qty: item.qty,
        };
      });

    const existTicket = await this.ticketService.ticketByUser(
      customer_name,
      customer_phone,
    );

    const insertTickets: QueueTicket[] = [];
    if (customer_name && customer_phone && existTicket) {
      existTicket.items.push(...items);
      await this.ticketService.updateQueueTicket(existTicket);

      existTicket.items = items;
      insertTickets.push(existTicket);
    } else {
      const ticket: QueueTicket = { ...customerOrder, items };
      const order: QueueOrder = { ...customerOrder, tickets: [ticket] };

      const queueOrder = await this.ticketService.createQueueOrder(order);
      insertTickets.push(...queueOrder.tickets);
    }

    if (
      transaction.customer_category?.has_vip_pass ||
      transaction.customer_type === TransactionUserType.VIP
    ) {
      insertTickets.forEach((ticket) => {
        const ticket_id = ticket.id;
        const items = {};
        ticket.items.forEach((item) => {
          const item_id = item['item_queue_id'];
          const currentItem = items[item_id];

          if (currentItem) {
            currentItem.qty += item.qty;
          }
          items[item_id] = currentItem
            ? currentItem
            : {
                item_id,
                ticket_id,
                qty: item.qty,
              };
        });

        Object.values(items).forEach((payload: any) => {
          this.create(payload);
        });
      });
    }
  }

  async create(data: RegisterQueueDto): Promise<void> {
    const queue = await this.queueService.getTicketItems(
      data.ticket_id,
      data.item_id,
    );
    const queueRequest: any = {
      qty: data.qty,
      item_id: queue.id,
      vip: true,
    };
    const registerQueueManager = new RegisterQueueManager(this.bucketService);
    registerQueueManager.setData(queueRequest);
    registerQueueManager.setService(this.queueService, TABLE_NAME.QUEUE);
    await registerQueueManager.execute();
  }
}

==> 20.2.10-alpha.1.tar.gz: pos-be/src/modules/queue/infrastructure/handlers/transaction.handler.ts <==
import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { TransactionDataService } from 'src/modules/transaction/transaction/data/services/transaction-data.service';
import {
  TransactionChangeStatusEvent,
  TransactionCreateQueueEvent,
} from 'src/modules/transaction/transaction/domain/entities/event/transaction-change-status.event';
import { TicketDataService } from '../../data/services/ticket.service';
import { QueueOrder } from '../../domain/entities/order.entity';
import { QueueTicket } from '../../domain/entities/ticket.entity';
import { QueueItem } from '../../domain/entities/queue-item.entity';
import * as moment from 'moment';

@EventsHandler(TransactionChangeStatusEvent, TransactionCreateQueueEvent)
export class QueueTransactionHandler
  implements IEventHandler<TransactionChangeStatusEvent>
{
  constructor(
    private dataService: TransactionDataService,
    private queueService: TicketDataService,
  ) {}

  async handle(event: TransactionChangeStatusEvent) {
    const process_data = event.data.data;

    /**
     * If data still in process (not settled) then don't create the queue order
     */
    if (process_data?.status != 'settled') return;

    const transaction = await this.dataService.getOneByOptions({
      where: {
        id: event.data.id,
      },
      relations: ['items'],
    });

    const date = transaction.booking_date ?? transaction.invoice_date;
    const queue_date = moment(date, 'YYYY-MM-DD').unix();

    const { id, customer_name, customer_phone, invoice_code } = transaction;

    const customerOrder = {
      code: invoice_code,
      customer: customer_name,
      phone: customer_phone,
      date: queue_date,
      transaction_id: id,
    };

    const items = transaction.items.map<QueueItem>((item) => {
      return {
        item_id: item.item_id,
        qty: item.qty,
      };
    });

    const ticket: QueueTicket = { ...customerOrder, items };
    const order: QueueOrder = { ...customerOrder, tickets: [ticket] };

    this.queueService.createQueueOrder(order);
  }
}
